############################################################################
# Unified Plone installer instance build script
# by Steve McMahon (steve at dcn.org)
#
# This script is meant to be invoked by another process
# that will choose the exec Python.
#
# $LastChangedDate: 2010-01-26 16:06:40 -0800 (Tue, 26 Jan 2010) $ $LastChangedRevision: 33471 $

import sys, os, os.path, stat, subprocess, shutil, iniparse, platform

# pick up globals from command line
(UIDIR,
PLONE_HOME,
INSTANCE_HOME,
CLIENT_USER,
ZEO_USER,
PASSWORD,
ROOT_INSTALL,
RUN_BUILDOUT,
OFFLINE,
ITYPE,
LOG_FILE) = sys.argv[1:]


def doCommand(command):
    po = subprocess.Popen(command,
                          shell=True,
                          universal_newlines=True,
                          stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    stdout, stderr = po.communicate()
    logfile = file(LOG_FILE, 'a')
    logfile.write(stdout)
    logfile.close()
    return po.returncode


print "Copying buildout skeleton"
shutil.copytree(os.path.join(UIDIR, 'base_skeleton'), INSTANCE_HOME)

# remove OS X and svn detritus (this is mainly helpful for installer development)
doCommand('find %s -name "._*" -exec rm {} \; > /dev/null' % INSTANCE_HOME)
doCommand('find %s -name ".svn" | xargs rm -rf' % INSTANCE_HOME)


#############################
# buildout.cfg customizations

# read appropriate buildout template
dest = os.path.join(INSTANCE_HOME, 'buildout.cfg')
if ITYPE == 'standalone':
    template = 'standalone.cfg'
else:
    template = 'cluster.cfg'
fd = file(os.path.join(UIDIR, 'buildout_templates', template))
buildout = fd.read()
fd.close()

### Skip during alpha
# # if standalone, non-root & OS X Leopard or later, install controller
# if (ITYPE == 'standalone') and \
#    (ROOT_INSTALL != '1') and \
#    (platform.system() == 'Darwin') and \
#    (int(platform.platform().split('-')[1].split('.')[0]) >= 9):
#     # add to parts
#     parts = buildout.buildout.parts.split()
#     parts.append('osxcontroller')
#     buildout.buildout.parts = '\n'.join(parts)
#     # add part
#     buildout.osxcontroller.recipe = 'plone.recipe.osxcontroller'

# set password
buildout = buildout.replace('__PASSWORD__', PASSWORD)

fn = os.path.join(INSTANCE_HOME, 'buildout.cfg')
fd = file(fn, 'w')
fd.write(buildout)
fd.close()
os.chmod(fn, stat.S_IRUSR | stat.S_IWUSR)


#############################
# base.cfg customizations

fd = file(os.path.join(UIDIR, 'buildout_templates', 'base.cfg'))
buildout = iniparse.INIConfig(fd)
fd.close()

mainClient = buildout.instance
client1 = buildout.client1
client2 = buildout.client2
zeoServer = buildout.zeoserver

# set buildout location
buildout.buildout['eggs-directory'] = '%s/buildout-cache/eggs' % PLONE_HOME
buildout.buildout['download-cache'] = '%s/buildout-cache/downloads' % PLONE_HOME
buildout.buildout['extends-cache'] = '%s/buildout-cache/downloads/extends' % PLONE_HOME

if ROOT_INSTALL == '1':
    # Set effective-user
    mainClient['effective-user'] = CLIENT_USER
    client1['effective-user'] = CLIENT_USER
    client2['effective-user'] = CLIENT_USER
    zeoServer['effective-user'] = ZEO_USER
    buildout.unifiedinstaller['sudo-command'] = 'sudo'
else:
    # remove effective-user options
    del mainClient['effective-user']
    del client1['effective-user']
    del zeoServer['effective-user']
    # remove chown commands
    for section in (buildout.chown, buildout['chown-zeo']):
        section.command = \
            '\n'.join( [s for s in section.command.split('\n') 
                          if len(s) and not s.count('chown')] )

if ITYPE == 'standalone':
    del buildout['zeoserver']
    del buildout['client1']
    del buildout['client2']
    del buildout['chown-zeo']
else:
    del buildout['instance']
    del buildout['chown']

fn = os.path.join(INSTANCE_HOME, 'base.cfg')
fd = file(fn, 'w')
fd.write(str(buildout))
fd.close()
os.chmod(fn, stat.S_IRUSR | stat.S_IWUSR)


print "Bootstrapping"
os.chdir(INSTANCE_HOME)
returncode = doCommand('"%s" bootstrap.py' % sys.executable)
if returncode:
    print "bootstrap failed, returned an error code: %s; Aborting." % returncode
    sys.exit(returncode)

################
# Start the fun!
if RUN_BUILDOUT == '1':
    print "Running buildout; this takes a while..."
    
    os.chdir(INSTANCE_HOME)
    
    if OFFLINE == '1':
        returncode = doCommand(os.path.join(INSTANCE_HOME, 'bin', 'buildout') + " -NU buildout:install-from-cache=true")
    else:
        returncode = doCommand(os.path.join(INSTANCE_HOME, 'bin', 'buildout') + " -NU")
    
    if returncode:
        print "Buildout returned an error code: %s; Aborting." % returncode
        sys.exit(returncode)

    if ITYPE == 'standalone':
        if not ( os.path.exists(os.path.join(INSTANCE_HOME, 'bin', 'instance')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'parts', 'instance')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'var')) ):
            print "Parts of the install are missing. Buildout must have failed. Aborting."
            sys.exit(1)
    else:
        if not ( os.path.exists(os.path.join(INSTANCE_HOME, 'bin', 'zeoserver')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'bin', 'client1')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'parts', 'client1')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'var')) ):
            print "Parts of the install are missing. Buildout must have failed. Aborting."
            sys.exit(1)

else:
    print "Skipping bin/buildout at your request."
